'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapFieldRules = exports.wrapFieldRulesConfig = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _titaniumRules = require('./titaniumRules');

var _titaniumRules2 = _interopRequireDefault(_titaniumRules);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

var wrapFieldRulesConfig = exports.wrapFieldRulesConfig = function () {
  function wrapFieldRulesConfig(input) {
    return input != null && typeof input.fieldName === 'string' && input.typeRules instanceof _titaniumRules2.default;
  }

  ;
  Object.defineProperty(wrapFieldRulesConfig, Symbol.hasInstance, {
    value: function value(input) {
      return wrapFieldRulesConfig(input);
    }
  });
  return wrapFieldRulesConfig;
}();

var wrapFieldRules = exports.wrapFieldRules = function wrapFieldRules(config) {
  if (!wrapFieldRulesConfig(config)) {
    throw new TypeError('Value of argument "config" violates contract.\n\nExpected:\nwrapFieldRulesConfig\n\nGot:\n' + _inspect(config));
  }

  var checkAccess = function checkAccess(node, context) {
    if (!config.typeRules.isReadable(node, context, config.fieldName)) {
      context.tql.throw(403, 'Access violation: Current viewer has no ' + ('privilege to \'read\' on field \'' + config.fieldName + '\''));
    }

    return true;
  };

  var wrappedFieldResolve = function () {
    var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(node, args, context, info) {
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              (0, _invariant2.default)(typeof context !== 'undefined', 'wrappedResolve expects \'context\' to be set at a single request');

              (0, _invariant2.default)(typeof context.user !== 'undefined', 'wrappedResolve expects an \'user\' field inside \'context\'');

              _context.next = 4;
              return checkAccess(node, context);

            case 4:
              if (!_context.sent) {
                _context.next = 8;
                break;
              }

              _context.next = 7;
              return config.fieldResolve(node, args, context, info);

            case 7:
              return _context.abrupt('return', _context.sent);

            case 8:
              return _context.abrupt('return', null);

            case 9:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined);
    }));

    return function wrappedFieldResolve(_x, _x2, _x3, _x4) {
      return _ref.apply(this, arguments);
    };
  }();

  return wrappedFieldResolve;
};

function _inspect(input, depth) {
  var maxDepth = 4;
  var maxKeys = 15;

  if (depth === undefined) {
    depth = 0;
  }

  depth += 1;

  if (input === null) {
    return 'null';
  } else if (input === undefined) {
    return 'void';
  } else if (typeof input === 'string' || typeof input === 'number' || typeof input === 'boolean') {
    return typeof input === 'undefined' ? 'undefined' : _typeof(input);
  } else if (Array.isArray(input)) {
    if (input.length > 0) {
      var _ret = function () {
        if (depth > maxDepth) return {
            v: '[...]'
          };

        var first = _inspect(input[0], depth);

        if (input.every(function (item) {
          return _inspect(item, depth) === first;
        })) {
          return {
            v: first.trim() + '[]'
          };
        } else {
          return {
            v: '[' + input.slice(0, maxKeys).map(function (item) {
              return _inspect(item, depth);
            }).join(', ') + (input.length >= maxKeys ? ', ...' : '') + ']'
          };
        }
      }();

      if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
    } else {
      return 'Array';
    }
  } else {
    var keys = Object.keys(input);

    if (!keys.length) {
      if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
        return input.constructor.name;
      } else {
        return 'Object';
      }
    }

    if (depth > maxDepth) return '{...}';
    var indent = '  '.repeat(depth - 1);
    var entries = keys.slice(0, maxKeys).map(function (key) {
      return (/^([A-Z_$][A-Z0-9_$]*)$/i.test(key) ? key : JSON.stringify(key)) + ': ' + _inspect(input[key], depth) + ';';
    }).join('\n  ' + indent);

    if (keys.length >= maxKeys) {
      entries += '\n  ' + indent + '...';
    }

    if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
      return input.constructor.name + ' {\n  ' + indent + entries + '\n' + indent + '}';
    } else {
      return '{\n  ' + indent + entries + '\n' + indent + '}';
    }
  }
}